<?php

use yii\helpers\Html;
use common\logic\Order\Order\Model\Order;
use common\helpers\Price;

$producerManager = $this->getProducerManager();
$userManager = $this->getUserManager();
$documentManager = $this->getDocumentManager();
$orderManager = $this->getOrderManager();

$displayPrices = Yii::$app->controller->getClass() != 'DeliveryNote' || (Yii::$app->controller->getClass() == 'DeliveryNote' && $producerManager->getConfig('document_display_prices_delivery_note'));
$displayProductDescription = $producerManager->getConfig('document_display_product_description');
$documentPriceDecimals = (int) $producerManager->getConfig('option_document_price_decimals');

?>

<div class="document-download">

    <div id="block-addresses">
        <div class="producer">
            <?php if (strlen($producer->logo)) : ?>
                <div class="logo">
                    <img style="max-height: 80px;" src="<?= $producerManager->getUrlLogo($producer) ?>"/>
                </div>
            <?php endif; ?>
            <div class="address"><?= $producerManager->getFullAddress($producer, true); ?></div>
        </div>
        <div class="user">
            <?php if ($document->address && strlen($document->address) > 0): ?>
                <?= nl2br($document->address) ?>
            <?php else: ?>
                <?= $userManager->getFullAddress($document->user, true); ?>
            <?php endif; ?>
        </div>
    </div>

    <div id="block-infos-document">
        <div class="date">
            Le <?= strftime('%d %B %Y', strtotime($document->date)) ?>
        </div>
        <div class="reference">
            <?php if (strlen($document->reference)) : ?>
                <?= $documentManager->getType($document); ?> N°<?= $document->reference; ?>
            <?php else: ?>
                <div class="block-is-draft"><?= $documentManager->getType($document); ?> non
                    validé<?= ($documentManager->getType($document) == 'Facture') ? 'e' : '' ?></div>
            <?php endif; ?>
        </div>
        <div class="name">
            <strong>Libellé : </strong><?= $document->name; ?>
        </div>
    </div>

    <?php if (strlen($document->comment)): ?>
        <div class="block-infos">
            <strong>Commentaire</strong><br/>
            <?= Html::encode($document->comment) ?>
        </div>
    <?php endif; ?>

    <div id="block-products">
        <?php if (count($document->orders) > 0) : ?>
            <table class="table table-bordered">
                <thead>
                <tr>
                    <th class="align-left">Produit</th>
                    <?php if ($displayPrices): ?>
                        <?php if ($producer->taxRate->value == 0): ?>
                            <th>Prix unitaire</th>
                        <?php else: ?>
                            <th>Prix unitaire HT</th>
                        <?php endif; ?>
                    <?php endif; ?>
                    <th>Quantité</th>
                    <th>Unité</th>
                    <?php if ($displayPrices): ?>
                        <?php if ($producer->taxRate->value == 0): ?>
                            <th>Prix</th>
                        <?php else: ?>
                            <th>TVA</th>
                            <th>Prix HT</th>
                        <?php endif; ?>
                    <?php endif; ?>

                </tr>
                </thead>
                <tbody>

                <?php if ($producerManager->isDocumentDisplayOrders($document)): ?>
                    <?php foreach ($document->orders as $order): ?>
                        <tr>
                            <td>
                                <strong><?= $orderManager->getOrderUsername($order); ?></strong>
                                <?php if ($order->distribution): ?>
                                    le <?= date('d/m/Y', strtotime($order->distribution->date)) ?>
                                <?php endif; ?>
                            </td>
                            <?php if ($displayPrices): ?>
                                <td class="align-center"></td>
                            <?php endif; ?>
                            <td></td>
                            <td></td>
                            <?php if ($displayPrices): ?>
                                <?php if ($producer->taxRate->value != 0): ?>
                                    <td class="align-center"></td>
                                <?php endif; ?>
                                <td class="align-center"></td>
                            <?php endif; ?>
                        </tr>
                        <?php foreach ($order->productOrder as $productOrder): ?>
                            <?= $this->render('_download_product_line', [
                                'producer' => $producer,
                                'document' => $document,
                                'productOrder' => $productOrder,
                                'displayOrders' => true,
                                'displayPrices' => $displayPrices,
                                'displayProductDescription' => $displayProductDescription,
                                'documentPriceDecimals' => $documentPriceDecimals
                            ]) ?>
                        <?php endforeach; ?>
                    <?php endforeach; ?>
                <?php else: ?>
                    <?php foreach ($documentManager->getProductsOrders($document) as $product): ?>
                        <?php foreach ($product as $productOrder): ?>
                            <?= $this->render('_download_product_line', [
                                'producer' => $producer,
                                'document' => $document,
                                'productOrder' => $productOrder,
                                'displayPrices' => $displayPrices,
                                'displayProductDescription' => $displayProductDescription,
                                'documentPriceDecimals' => $documentPriceDecimals
                            ]) ?>
                        <?php endforeach; ?>
                    <?php endforeach; ?>
                <?php endif; ?>
                <?php if ($displayPrices): ?>
                    <?php $typeAmount = $documentManager->isInvoicePrice($document) ? Order::INVOICE_AMOUNT_TOTAL : Order::AMOUNT_TOTAL; ?>

                    <?php if ($producer->taxRate->value != 0): ?>

                        <tr>
                            <td class="align-right" colspan="5"><strong>Total HT</strong></td>
                            <td class="align-center">
                                <?= Price::format($documentManager->getAmount($document, $typeAmount)); ?>
                            </td>
                        </tr>

                        <?php
                        $taxRateArray = $this->getTaxRateManager()->findTaxRatesAsArray();
                        foreach ($documentManager->getTotalVatArray($document, $typeAmount) as $idTaxRate => $totalVat): ?>
                            <tr>
                                <td class="align-right" colspan="5">
                                    <strong>TVA <?= $taxRateArray[$idTaxRate]->value * 100 ?> %</strong></td>
                                <td class="align-center">
                                    <?= Price::format($totalVat); ?>
                                </td>
                            </tr>
                        <?php endforeach; ?>

                        <!--<tr>
                                                <td class="align-right" colspan="5"><strong>TVA</strong></td>
                                                <td class="align-center">
                                                    <?= Price::format($documentManager->getAmountWithTax($document, $typeAmount) - $documentManager->getAmount($document, $typeAmount)) ?>
                                                </td>
                                        </tr>-->
                        <tr>
                            <td class="align-right" colspan="5"><strong>Total TTC</strong></td>
                            <td class="align-center"><?= Price::format($documentManager->getAmountWithTax($document, $typeAmount)) ?></td>
                        </tr>
                    <?php else: ?>
                        <tr>
                            <td class="align-right" colspan="4">
                                <strong>Total</strong><br/>
                                TVA non applicable
                            </td>
                            <td class="align-center"><?= Price::format($documentManager->getAmount($document, $typeAmount)) ?></td>
                        </tr>
                    <?php endif; ?>
                <?php endif; ?>
                </tbody>
            </table>
        <?php else : ?>
            <div id="block-no-product">
                <strong>Aucun produit</strong>
            </div>
        <?php endif; ?>
    </div>

    <?php
    $fieldProducerDocumentInfo = 'document_infos_' . str_replace('deliverynote', 'delivery_note', strtolower($documentManager->getClass($document))); ?>
    <?php if (strlen($producer->$fieldProducerDocumentInfo)): ?>
        <div class="block-infos">
            <strong>Informations</strong><br/>
            <?= nl2br(Html::encode($producer->$fieldProducerDocumentInfo)) ?>
        </div>
    <?php endif; ?>
</div>