<?php

namespace common\logic\PointSale\PointSale\Repository;

use common\logic\AbstractRepository;
use common\logic\Distribution\Distribution\Model\Distribution;
use common\logic\PointSale\PointSale\Model\PointSale;
use common\logic\Producer\Producer\Model\Producer;
use common\logic\User\User\Model\User;

class PointSaleRepository extends AbstractRepository
{
    protected PointSaleRepositoryQuery $query;

    public function loadDependencies(): void
    {
        $this->loadQuery(PointSaleRepositoryQuery::class);
    }

    public function getDefaultOptionsSearch(): array
    {
        return [
            self::WITH => [],
            self::JOIN_WITH => [],
            self::ORDER_BY => 'is_bread_box ASC, name ASC',
            self::ATTRIBUTE_ID_PRODUCER => 'point_sale.id_producer'
        ];
    }

    public function findOnePointSaleById(int $id): ?PointSale
    {
        return $this->createDefaultQuery()
            ->filterById($id)
            ->findOne();
    }

    public function findOnePointSaleByIdUser(int $idUser): ?PointSale
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->filterByIdUser($idUser)
            ->findOne();
    }

    public function findOnePointSaleByName(string $name): ?PointSale
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->filterByName($name)
            ->fineOne();
    }

    public function findPointSalesByDistribution(Distribution $distribution)
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->with(['pointSaleDistribution' => function ($q) use ($distribution) {
                $q->where(['id_distribution' => $distribution->id]);
            }])
            ->find();
    }

    /**
     * Retourne le nombre de points de vente pour le producteur courant.
     */
    public function countPointSales(): int
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->count();
    }

    public function findPointSales()
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->find();
    }

    public function queryPointSalesPublic(Producer $producer)
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->filterByRestrictedAccess(0)
            ->orderBy('code ASC, restricted_access ASC, is_bread_box ASC, `default` DESC, name ASC')
            ->query();
    }

    public function populatePointSaleDropdownList(): array
    {
        $pointSalesArrayDropdown = ['' => '--'];
        $pointSalesArray = $this->findPointSales();

        foreach ($pointSalesArray as $pointSale) {
            $pointSalesArrayDropdown[$pointSale['id']] = $pointSale['name'];
        }

        return $pointSalesArrayDropdown;
    }

    public function findByBillingUser(User $user)
    {
        return $this->createDefaultQuery()
            ->filterIsOnline()
            ->filterByIdUser($user->id)
            ->orderBy('name ASC')
            ->find();
    }
}