<?php

namespace common\logic\User\User\Service;

use common\helpers\GlobalParam;
use common\helpers\Mailjet;
use common\logic\AbstractService;
use common\logic\Producer\Producer\Model\Producer;
use common\logic\User\User\Model\User;
use common\logic\UtilsInterface;

class UserUtils extends AbstractService implements UtilsInterface
{
    protected UserSolver $userSolver;

    public function loadDependencies(): void
    {
        $this->userSolver = $this->loadService(UserSolver::class);
    }

    public function sendEmailSignup(User $user, Producer $producer)
    {
        Mailjet::sendMail([
            'from_email' => \Yii::$app->params['adminEmail'],
            'from_name' => 'Opendistrib',
            'to_email' => $user->email,
            'to_name' => $this->userSolver->getUsername($user),
            'subject' => '[Opendistrib] Inscription',
            'content_view_text' => '@common/mail/signup-text.php',
            'content_view_html' => '@common/mail/signup-html.php',
            'content_params' => [
                'user' => $user,
                'producer' => $producer
            ]
        ]);
    }

    /**
     * Envoie un email de bienvenue à l'utilisateur lors de son inscription
     * via le backend du site.
     *
     */
    public function sendMailWelcome(User $user, string $password): void
    {
        if (strlen($user->email)) {
            $producer = Producer::findOne(GlobalParam::getCurrentProducerId());
            \Yii::$app->mailer->compose();
            $mail = \Yii::$app->mailer->compose(
                ['html' => 'createUserAdmin-html', 'text' => 'createUserAdmin-text'], ['user' => $user, 'producer' => $producer, 'password' => $password]
            )
                ->setTo($user->email)
                ->setFrom(['contact@opendistrib.net' => 'distrib'])
                ->setSubject('[Opendistrib] Inscription')
                ->send();
        }
    }
}