<?php

/**
 * Copyright distrib (2018)
 *
 * contact@opendistrib.net
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace backend\controllers;

use common\logic\Document\Invoice\Model\InvoiceSearch;
use common\logic\Order\Order\Model\Order;

class InvoiceController extends DocumentController
{
    /**
     * Liste les factures
     */
    public function actionIndex()
    {
        $searchModel = new InvoiceSearch();
        $dataProvider = $searchModel->search(\Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionAjaxDeleteDeliveryNote($idInvoice, $idDeliveryNote)
    {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;

        $invoiceManager = $this->getInvoiceManager();

        $invoice = $invoiceManager->findOneInvoiceById($idInvoice);

        if ($invoice && $invoiceManager->isStatusDraft($invoice)) {
            Order::updateAll([
                'id_invoice' => null
            ], [
                'id_delivery_note' => $idDeliveryNote
            ]);

            return [
                'alert' => [
                    'type' => 'success',
                    'message' => 'Bon de livraison supprimé de la facture.'
                ]
            ];
        } else {
            return [
                'alert' => [
                    'type' => 'error',
                    'message' => 'Une erreur est survenue lors de la suppression du bon de livraison.'
                ]
            ];
        }
    }

    public function actionAjaxAddDeliveryNote($idInvoice, $idDeliveryNote)
    {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;

        $invoiceManager = $this->getInvoiceManager();
        $deliveryNoteManager = $this->getDeliveryNoteManager();

        $invoice = $invoiceManager->findOneInvoiceById($idInvoice);
        $deliveryNote = $deliveryNoteManager->findOneDeliveryNoteById($idDeliveryNote);

        if ($invoice && $invoiceManager->isStatusDraft($invoice) && $deliveryNote) {
            Order::updateAll([
                'id_invoice' => $idInvoice
            ], [
                'id_delivery_note' => $idDeliveryNote
            ]);

            return [
                'alert' => [
                    'type' => 'success',
                    'message' => 'Bon de livraison ajouté à la facture.'
                ]
            ];
        } else {
            return [
                'alert' => [
                    'type' => 'error',
                    'message' => 'Une erreur est survenue lors de l\'ajout du bon de livraison.'
                ]
            ];
        }
    }

}
