<?php

/**
 * Copyright distrib (2018)
 *
 * contact@opendistrib.net
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace common\logic\Document\Document\Model;

use common\components\ActiveRecordCommon;
use common\logic\Order\Order\Model\Order;
use common\logic\Order\Order\Repository\OrderRepository;
use common\logic\Producer\Producer\Model\Producer;
use common\logic\User\User\Model\User;

class Document extends ActiveRecordCommon implements DocumentInterface
{
    const STATUS_DRAFT = 'draft';
    const STATUS_VALID = 'valid';

    const TAX_CALCULATION_METHOD_SUM_OF_ROUNDINGS = 'sum-roundings';
    const TAX_CALCULATION_METHOD_ROUNDING_OF_THE_SUM = 'rounding-sum';
    const TAX_CALCULATION_METHOD_DEFAULT = self::TAX_CALCULATION_METHOD_ROUNDING_OF_THE_SUM;

    public static $taxCalculationMethodArray = [
        self::TAX_CALCULATION_METHOD_ROUNDING_OF_THE_SUM => 'Arrondi de la somme des lignes',
        self::TAX_CALCULATION_METHOD_SUM_OF_ROUNDINGS => 'Somme des arrondis de chaque ligne'
    ];

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name', 'id_user'], 'required'],
            [['date'], 'safe'],
            [['comment', 'address', 'tax_calculation_method'], 'string'],
            [['id_user', 'id_producer'], 'integer'],
            [['is_sent'], 'boolean'],
            [['name', 'reference', 'status'], 'string', 'max' => 255],
            [['deliveryNotes'], 'safe']
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Nom',
            'reference' => 'Référence',
            'date' => 'Date',
            'comment' => 'Commentaire',
            'id_user' => 'Utilisateur',
            'address' => 'Adresse',
            'id_producer' => 'Producteur',
            'status' => 'Statut',
            'tax_calculation_method' => 'Méthode de calcul de la TVA',
            'is_sent' => 'Envoyé'
        ];
    }

    /*
     * Relations
     */

    public function getUser()
    {
        return $this->hasOne(User::class, ['id' => 'id_user']);
    }

    public function populateUser(User $user): void
    {
        $this->populateFieldObject('id_user', 'user', $user);
    }

    public function getProducer()
    {
        return $this->hasOne(Producer::class, ['id' => 'id_producer']);
    }

    public function populateProducer(Producer $producer): void
    {
        $this->populateFieldObject('id_producer', 'producer', $producer);
    }

    public function relationOrders($fieldIdDocument)
    {
        $orderRepository = OrderRepository::getInstance();
        $getDefaultOptionsSearch = $orderRepository->getDefaultOptionsSearch();

        return $this->hasMany(Order::class, [$fieldIdDocument => 'id'])
            ->with($getDefaultOptionsSearch['with'])
            ->joinWith($getDefaultOptionsSearch['join_with'])
            ->orderBy('distribution.date ASC');
    }
}
